.ifdef mcu_8535

;;; read_eeprom ROUTINE ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This routine fills the settings and DMX output table using fixed values.

read_eeprom:
; Zeroise temporary bit storage register
    CLR     temp3
; Store current transmission enabled state
    SBIC    UCR,TXEN
; Set transmission was enabled flag
    SBR     temp3,1<<2

; Store data in temporary register
    LDI     temp1,'R'
; Check whether reception enabled status is 'R'
    CPI     temp1,'R'
    BRNE    read_eeprom_disable_reception
; Reception should be enabled
    SBR     temp3,1<<0
read_eeprom_disable_reception:

; Store data in temporary register
    LDI     temp1,'T'
; Check whether transmission enabled status is 'T'
    CPI     temp1,'T'
    BRNE    read_eeprom_disable_trans
; Transmission should be enabled
    SBR     temp3,1<<1
read_eeprom_disable_trans:

; Store data in temporary register
    LDI     temp1,'E'
; Check whether mux enabled status is 'E'
    CPI     temp1,'E'
    BRNE    read_eeprom_disable_mux
; Set mux flag (bit in flags register)
    SET
    BLD     flags,mux_status
    RJMP    read_eeprom_mux_finished
read_eeprom_disable_mux:
; Unset mux flag (bit in flags register)
    CLT
    BLD     flags,mux_status
read_eeprom_mux_finished:

; Temporarily store merge status
    LDI     temp1,'x'
; Check whether merge should be enabled and update the corresponding flag
    CPI     temp1,'X'
    BRNE    disable_merge
; Set merge flag (bit in flags register)
    SET
    RJMP    update_merge
disable_merge:
; Unset merge flag (bit in flags register)
    CLT
update_merge:
; Update actual merge flag (bit in flags register)
    BLD     flags,merge_status

; Store new break time
    LDI     temp1,256 - default_break_time + 2
    MOV     break_time,temp1

; Store new mab time
    LDI     temp1,default_mab_time - 1
    MOV     mab_time,temp1

; Store new output count
    LDI     temp1,255
    MOV     output_count,temp1

; Store new mux address
    LDI     temp1,0
    MOV     mux_address,temp1

; Store new input startcode to listen for
    LDI     temp1,0
    MOV     input_startcode,temp1

; Temporarily store input wildcard status
    LDI     temp1,'#'
; Check whether the input wildcard should be enabled and update the corresponding flag
    CPI     temp1,'*'
    BRNE    disable_input_wildcard
; Set input_wildcard flag (bit 1 in flags register)
    SET
    RJMP    update_input_wildcard
disable_input_wildcard:
; Unset input_wildcard flag (bit 1 in flags register)
    CLT
update_input_wildcard:
; Update actual wildcard flag (bit 1 in flags register)
    BLD     flags,input_wildcard

; Temporarily store mux invert status
    LDI     temp1,'N'
; Check whether the mux inverting should be enabled and update the corresponding flag
    CPI     temp1,'N'
    BRNE    disable_mux_invert
; Set mux_invert flag (bit 3 in flags register)
    SET
    RJMP    update_mux_invert
disable_mux_invert:
; Unset mux_invert flag (bit 3 in flags register)
    CLT
update_mux_invert:
; Update actual mux invert flag (bit 3 in flags register)
    BLD     flags,mux_invert

; Store mass output minimum high value
    LDI     temp1,high(output_min + 1)
    MOV     mass_output_min_h,temp1

; Store mass output minimum low value
    LDI     temp1,low(output_min + 1)
    MOV     mass_output_min_l,temp1

; Store mass output maximum high value
    LDI     temp1,high(output_max + 1)
    MOV     mass_output_max_h,temp1

; Store mass output maximum low value
    LDI     temp1,low(output_max + 1)
    MOV     mass_output_max_l,temp1

; Fill up temporary pointer with address of DMX output table
    LDI     ptr_temp_h,high(output_min)
    LDI     ptr_temp_l,low(output_min)
read_eeprom_loop:
; Store data from EEPROM data register
    LDI     temp1,0
    ST      Z+,temp1
    CPI     ptr_temp_h,high(output_max+1)
    LDI     temp1,low(output_max+1)
    CPC     ptr_temp_l,temp1
    BRNE    read_eeprom_loop

; Let reception enabled setting take effect
    SBRC    temp3,1<<0
; Enable reception
    SBI     UCR,RXEN
; Let transmission enabled setting take effect
    SBRC    temp3,1<<1
; Transmission must be enabled, however only do so if transmission wasn't enabled yet,
; by checking the transmission was enabled flag
    SBRS    temp3,1<<2
; Transmit initial BREAK, MAB and initial value (start code), this will also enable transmision
    RCALL   transmit_break_mab
    RET

.endif

.ifndef mcu_8535

;;; read_eeprom ROUTINE ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This routine reads output data from the EEPROM and
; fills the settings and DMX output table with it.

read_eeprom:
; Zeroise temporary bit storage register
    CLR     temp3
; Store current transmission enabled state
    SBIC    UCR,TXEN
; Set transmission was enabled flag
    SBR     temp3,1<<2
; Wait for EEWE bit in EECR to go low, indicating that there is no EEPROM write operation going on
read_eeprom_wait_ready:
    SBIC    EECR,EEWE
    RJMP    read_eeprom_wait_ready

; Initialize EEPROM address; address 0 contains no data as it might cause problems according to the errata
    LDI     temp1,high(rom_input_status)
    LDI     temp2,low(rom_input_status)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read reception enabled status from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store data in temporary register
    IN      temp1,EEDR
; Check whether reception enabled status is 'R'
    CPI     temp1,'R'
    BRNE    read_eeprom_disable_reception
; Reception should be enabled
    SBR     temp3,1<<0
read_eeprom_disable_reception:

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_output_status)
    LDI     temp2,low(rom_output_status)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read transmission enabled status from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store data in temporary register
    IN      temp1,EEDR
; Check whether transmission enabled status is 'T'
    CPI     temp1,'T'
    BRNE    read_eeprom_disable_trans
; Transmission should be enabled
    SBR     temp3,1<<1
read_eeprom_disable_trans:

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mux_status)
    LDI     temp2,low(rom_mux_status)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read mux enabled status from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store data in temporary register
    IN      temp1,EEDR
; Check whether mux enabled status is 'E'
    CPI     temp1,'E'
    BRNE    read_eeprom_disable_mux
; Set mux flag (bit in flags register)
    SET
    BLD     flags,mux_status
    RJMP    read_eeprom_mux_finished
read_eeprom_disable_mux:
; Unset mux flag (bit in flags register)
    CLT
    BLD     flags,mux_status
read_eeprom_mux_finished:

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_merge_status)
    LDI     temp2,low(rom_merge_status)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read merge status from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Temporarily store merge status
    IN      temp1,EEDR
; Check whether merge should be enabled and update the corresponding flag
    CPI     temp1,'X'
    BRNE    disable_merge
; Set merge flag (bit in flags register)
    SET
    RJMP    update_merge
disable_merge:
; Unset merge flag (bit in flags register)
    CLT
update_merge:
; Update actual merge flag (bit in flags register)
    BLD     flags,merge_status

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_break_time)
    LDI     temp2,low(rom_break_time)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read break time from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store new break time
    IN      break_time,EEDR

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mab_time)
    LDI     temp2,low(rom_mab_time)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read mark after break (MAB) time from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store new mab time
    IN      mab_time,EEDR

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_output_count)
    LDI     temp2,low(rom_output_count)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read output count from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store new output count
    IN      output_count,EEDR

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mux_address)
    LDI     temp2,low(rom_mux_address)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read mux address from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store new mux address
    IN      mux_address,EEDR

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_input_startcode)
    LDI     temp2,low(rom_input_startcode)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read mux address from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store new input startcode to listen for
    IN      input_startcode,EEDR

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_input_wildcard)
    LDI     temp2,low(rom_input_wildcard)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read input wildcard status from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Temporarily store input wildcard status
    IN      temp1,EEDR
; Check whether the input wildcard should be enabled and update the corresponding flag
    CPI     temp1,'*'
    BRNE    disable_input_wildcard
; Set input_wildcard flag (bit 1 in flags register)
    SET
    RJMP    update_input_wildcard
disable_input_wildcard:
; Unset input_wildcard flag (bit 1 in flags register)
    CLT
update_input_wildcard:
; Update actual wildcard flag (bit 1 in flags register)
    BLD     flags,input_wildcard

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mux_invert)
    LDI     temp2,low(rom_mux_invert)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read mux invert status from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Temporarily store mux invert status
    IN      temp1,EEDR
; Check whether the mux inverting should be enabled and update the corresponding flag
    CPI     temp1,'N'
    BRNE    disable_mux_invert
; Set mux_invert flag (bit 3 in flags register)
    SET
    RJMP    update_mux_invert
disable_mux_invert:
; Unset mux_invert flag (bit 3 in flags register)
    CLT
update_mux_invert:
; Update actual mux invert flag (bit 3 in flags register)
    BLD     flags,mux_invert

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mass_output_min_h)
    LDI     temp2,low(rom_mass_output_min_h)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read mass output minimum high value from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store mass output minimum high value
    IN      mass_output_min_h,EEDR

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mass_output_min_l)
    LDI     temp2,low(rom_mass_output_min_l)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read mass output minimum low value from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store mass output minimum low value
    IN      mass_output_min_l,EEDR

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mass_output_max_h)
    LDI     temp2,low(rom_mass_output_max_h)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read mass output maximum high value from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store mass output maximum high value
    IN      mass_output_max_h,EEDR

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mass_output_max_l)
    LDI     temp2,low(rom_mass_output_max_l)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read mass output maximum low value from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store mass output maximum low value
    IN      mass_output_max_l,EEDR

; Initialize EEPROM address for channel values; bytes 1 up to 99 are reserved for storing individual bytes
    LDI     temp1,high(rom_channels - 1)
    LDI     temp2,low(rom_channels - 1)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Fill up temporary pointer with address of DMX output table
    LDI     ptr_temp_h,high(output_min)
    LDI     ptr_temp_l,low(output_min)
read_eeprom_loop:
; Increment EEPROM address
    IN      temp2,EEARL
    IN      temp1,EEARH
    INC     temp2
; Check whether the INC caused an overflow
    BRNE    read_eeprom_no_overflow
; There was an overflow
    INC     temp1
read_eeprom_no_overflow:
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read data from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store data from EEPROM data register
    IN      temp1,EEDR
    ST      Z+,temp1
    CPI     ptr_temp_h,high(output_max+1)
    LDI     temp1,low(output_max+1)
    CPC     ptr_temp_l,temp1
    BRNE    read_eeprom_loop
; Let reception enabled setting take effect
    SBRC    temp3,1<<0
; Enable reception
    SBI     UCR,RXEN
; Let transmission enabled setting take effect
    SBRC    temp3,1<<1
; Transmission must be enabled, however only do so if transmission wasn't enabled yet,
; by checking the transmission was enabled flag
    SBRS    temp3,1<<2
; Transmit initial BREAK, MAB and initial value (start code), this will also enable transmision
    RCALL   transmit_break_mab
    RET

.endif
